package com.kakao.sdk.navi.sample;

import android.content.Intent;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;

import com.kakao.kakaonavi.Destination;
import com.kakao.kakaonavi.Location;
import com.kakao.kakaonavi.KakaoNaviParams;
import com.kakao.kakaonavi.KakaoNaviService;
import com.kakao.kakaonavi.NaviOptions;
import com.kakao.kakaonavi.options.CoordType;
import com.kakao.kakaonavi.options.RpOption;
import com.kakao.kakaonavi.options.VehicleType;

import java.util.ArrayList;
import java.util.List;

public class KakaoNaviMainActivity extends AppCompatActivity implements View.OnClickListener, AdapterView.OnItemSelectedListener {

    private static final int GO_SHARE = 1;
    private static final int GO_GUIDE = 2;

    public static final int DEST_REQ_CODE = 1001;
    public static final int START_REQ_CODE = 1002;
    public static final int STOP1_REQ_CODE = 1003;
    public static final int STOP2_REQ_CODE = 1004;
    public static final int STOP3_REQ_CODE = 1005;

    private int currentMethod;

    private Destination destination;
    private Location start;
    private Location stop1;
    private Location stop2;
    private Location stop3;

    private EditText destinationText;
    private EditText startText;
    private EditText stop1Text;
    private EditText stop2Text;
    private EditText stop3Text;
    private EditText startAngleText;


    private NaviOptions.Builder optionBuilder;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_kakaonavi_main);


        findViewById(R.id.radio_share_dest).setOnClickListener(this);
        findViewById(R.id.radio_navi_guide).setOnClickListener(this);

        Spinner vehicleTypeSpinner = (Spinner) findViewById(R.id.spinner_vehicle_type);
        vehicleTypeSpinner.setAdapter(new ArrayAdapter<VehicleType>(this, android.R.layout.simple_spinner_dropdown_item, VehicleType.values()));
        vehicleTypeSpinner.setOnItemSelectedListener(this);

        Spinner rpOptionSpinner = (Spinner) findViewById(R.id.spinner_rp_option);
        rpOptionSpinner.setAdapter(new ArrayAdapter<RpOption>(this, android.R.layout.simple_spinner_dropdown_item, RpOption.values()));
        rpOptionSpinner.setOnItemSelectedListener(this);

        destinationText = (EditText) findViewById(R.id.edit_destination);
        startText = (EditText) findViewById(R.id.edit_start_location);
        stop1Text = (EditText) findViewById(R.id.edit_stop1);
        stop2Text = (EditText) findViewById(R.id.edit_stop2);
        stop3Text = (EditText) findViewById(R.id.edit_stop3);
        startAngleText = (EditText) findViewById(R.id.edit_angle);

        destinationText.setOnClickListener(this);
        startText.setOnClickListener(this);
        stop1Text.setOnClickListener(this);
        stop2Text.setOnClickListener(this);
        stop3Text.setOnClickListener(this);

        Spinner routeInfoSpinner = (Spinner) findViewById(R.id.spinner_route_info);
        Spinner returnUriSpinner = (Spinner) findViewById(R.id.spinner_return_uri);
        routeInfoSpinner.setAdapter(new ArrayAdapter<Boolean>(this, android.R.layout.simple_spinner_dropdown_item, new Boolean[] { true, false}));
        returnUriSpinner.setAdapter(new ArrayAdapter<Boolean>(this, android.R.layout.simple_spinner_dropdown_item, new Boolean[] { true, false}));
        routeInfoSpinner.setOnItemSelectedListener(this);
        returnUriSpinner.setOnItemSelectedListener(this);
        routeInfoSpinner.setSelection(1);

        currentMethod = GO_SHARE;

        Button kakaoNaviButton = (Button) findViewById(R.id.kakao_navi_button);

        kakaoNaviButton.setOnClickListener(this);

        optionBuilder = NaviOptions.newBuilder();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.radio_share_dest:
                findViewById(R.id.layout_option).setVisibility(View.GONE);
                findViewById(R.id.layout_via_list).setVisibility(View.GONE);
                currentMethod = GO_SHARE;
                break;
            case R.id.radio_navi_guide:
                findViewById(R.id.layout_option).setVisibility(View.VISIBLE);
                findViewById(R.id.layout_via_list).setVisibility(View.VISIBLE);
                currentMethod = GO_GUIDE;
                break;
            case R.id.edit_destination:
            case R.id.edit_start_location:
            case R.id.edit_stop1:
            case R.id.edit_stop2:
            case R.id.edit_stop3:
                findLocation(v);
                break;
            case R.id.kakao_navi_button:
                executeKakaoNavi();
            default:
                break;
        }
    }

    private void findLocation(View view) {
        int requestCode;
        switch (view.getId()) {
            case R.id.edit_destination:
                requestCode = DEST_REQ_CODE;
                break;
            case R.id.edit_start_location:
                requestCode = START_REQ_CODE;
                break;
            case R.id.edit_stop1:
                requestCode = STOP1_REQ_CODE;
                break;
            case R.id.edit_stop2:
                requestCode = STOP2_REQ_CODE;
                break;
            case R.id.edit_stop3:
                requestCode = STOP3_REQ_CODE;
                break;
            default:
                return;
        }
        startActivityForResult(new Intent(this, KakaoNaviMapActivity.class), requestCode);
    }

    private void processLocationSearch(final int requestCode, final String name, final double x, final double y) {
        Location location = Location.newBuilder(name, x, y).build();
        switch (requestCode) {
            case DEST_REQ_CODE:
                destination = Destination.newBuilder(name, x, y).build();
                destinationText.setText(destination.getName());
                break;
            case START_REQ_CODE:
                start = location;
                startText.setText(location.getName());
                break;
            case STOP1_REQ_CODE:
                stop1 = location;
                stop1Text.setText(location.getName());
                break;
            case STOP2_REQ_CODE:
                stop2 = location;
                stop2Text.setText(location.getName());
                break;
            case STOP3_REQ_CODE:
                stop3 = location;
                stop3Text.setText(location.getName());
                break;
            default:
                break;
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case DEST_REQ_CODE:
            case START_REQ_CODE:
            case STOP1_REQ_CODE:
            case STOP2_REQ_CODE:
            case STOP3_REQ_CODE:
                if (data == null)
                    return;
                String name = data.getStringExtra("name");
                double x = data.getDoubleExtra("x", 0);
                double y = data.getDoubleExtra("y", 0);
                processLocationSearch(requestCode, name, x, y);
                break;

        }
    }

    private void executeKakaoNavi() {
        if (currentMethod == GO_SHARE) {
            requestShareDestination();
        } else if (currentMethod == GO_GUIDE) {
            requestNaviGuide();
        }
    }

    private void requestShareDestination() {
        KakaoNaviParams.Builder builder = KakaoNaviParams.newBuilder(destination).setNaviOptions(NaviOptions.newBuilder().setCoordType(CoordType.WGS84).build());
        KakaoNaviService.shareDestination(this, builder.build());
    }

    private void requestNaviGuide() {
        optionBuilder.setCoordType(CoordType.WGS84);

        if (start != null) {
            optionBuilder.setStartX(start.getX()).setStartY(start.getY());
        }

        String angle = startAngleText.getText().toString();
        if (!angle.isEmpty()) {
            optionBuilder.setStartAngle(Integer.parseInt(angle));
        }

        KakaoNaviParams.Builder builder = KakaoNaviParams.newBuilder(destination).setNaviOptions(optionBuilder.build());

        List<Location> viaLists = new ArrayList<Location>();
        if (stop1 != null)
            viaLists.add(stop1);
        if (stop2 != null)
            viaLists.add(stop2);
        if (stop3 != null)
            viaLists.add(stop3);

        if (!viaLists.isEmpty()) {
            builder.setViaList(viaLists);
        }

        KakaoNaviService.navigate(this, builder.build());
    }

    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        Spinner spinner = (Spinner) parent;
        switch (spinner.getId()) {
            case R.id.spinner_vehicle_type:
                optionBuilder.setVehicleType(VehicleType.values()[position]);
                break;
            case R.id.spinner_rp_option:
                optionBuilder.setRpOption(RpOption.values()[position]);
                break;
            case R.id.spinner_route_info:
                optionBuilder.setRouteInfo(position == 0);
                break;
            case R.id.spinner_return_uri:
                optionBuilder.setReturnUri(position == 0 ? "com.kakao.sdk.navi.sample.KakaoNaviMapActivity" : null);
                break;
            default:
                break;
        }
    }

    @Override
    public void onNothingSelected(AdapterView<?> parent) {

    }
}
