package com.kakao.kakaolink.v4.network;

import android.net.Uri;

import com.kakao.kakaolink.internal.KakaoTalkLinkProtocol;
import com.kakao.network.KakaoRequest;
import com.kakao.network.RequestConfiguration;
import com.kakao.network.ServerProtocol;
import com.kakao.util.helper.log.Logger;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Map;

/**
 * Template validation request class.
 * Created by kevin.kang on 2016. 11. 25..
 */

public class TemplateValidateRequest extends KakaoRequest {
    private String templateId;
    private Map<String, String> templateArgs;

    public TemplateValidateRequest(final RequestConfiguration configuration, final String templateId, final Map<String, String> tempalteArgs) {
        super(configuration);
        this.templateId = templateId;
        this.templateArgs = tempalteArgs;
    }

    public String getTemplateId() {
        return templateId;
    }

    public Map getTemplateArgs() {
        return templateArgs;
    }

    public String getTemplateArgsString() {
        if (templateArgs == null || templateArgs.isEmpty()) {
            return null;
        }
        JSONObject argsJson = new JSONObject();
        try {
            for (Map.Entry<String, String> entry : templateArgs.entrySet()) {
                argsJson.put(entry.getKey(), entry.getValue());
            }
            return argsJson.toString();
        } catch (JSONException e) {
            Logger.e(e.toString());
        }
        return null;
    }

    @Override
    public String getMethod() {
        return POST;
    }

    @Override
    public String getUrl() {
        Uri.Builder builder = new Uri.Builder();
        builder.scheme(ServerProtocol.SCHEME);
        builder.authority(ServerProtocol.API_AUTHORITY);
        builder.path(ServerProtocol.LINK_TEMPLATE_VALIDATE_PATH);
        return builder.build().toString();
    }

    @Override
    public Map<String, String> getParams() {
        Map<String, String> params = super.getParams();
        params.put("link_ver", "4.0");
        params.put("app_key", getAppKey());
        params.put(KakaoTalkLinkProtocol.TEMPLATE_ID, templateId);
        if (templateArgs != null && !templateArgs.isEmpty()) {
            params.put("template_args", getTemplateArgsString());
        }
        return params;
    }
}
